From: Christian Heller <c.heller@plomlompom.de>
Date: Sat, 16 Aug 2014 18:14:31 +0000 (+0200)
Subject: Server: Always display consumables on top of stack of inanimate objects.
X-Git-Tag: tce~659
X-Git-Url: https://plomlompom.com/repos/%7B%7Bprefix%7D%7D/static/%7B%7Bdb.prefix%7D%7D/conditions?a=commitdiff_plain;h=b01c00f25907d81051b38d9dc39b6781f087ea90;p=plomrogue

Server: Always display consumables on top of stack of inanimate objects.
---

diff --git a/TODO b/TODO
index 3953ade..a5eb094 100644
--- a/TODO
+++ b/TODO
@@ -9,10 +9,6 @@ BOTH SERVER/CLIENT:
 - make server and client communicate by specific world state info requests 
   in server/out, replacing server/worldstate
 
-SERVER
-
-- fix: AI knows about consumables not on top of stack while player doesn't
-
 CLIENT:
 
 - re-work unnecessary complex command / keybinding / server message mapping
diff --git a/src/server/things.h b/src/server/things.h
index 4c21c45..ea37a81 100644
--- a/src/server/things.h
+++ b/src/server/things.h
@@ -18,7 +18,7 @@ struct Thing
     uint8_t id;                  /* individual thing's unique identifier */
     struct Thing * owns;         /* chain of things owned / in inventory */
     struct yx_uint8 pos;         /* coordinate on map */
-    char * fov_map;              /* map of the thing's field of view */
+    char * fov_map;              /* thing's FOV map; 'v':visible, 'H':hidden */
     char * mem_map;              /* map knowledge of thing by FOV and memory */
     uint8_t type;                /* ID of appropriate thing definition */
     uint8_t lifepoints;          /* 0: thing is inanimate; >0: hitpoints */
diff --git a/src/server/io.c b/src/server/io.c
index 947c146..14b5e48 100644
--- a/src/server/io.c
+++ b/src/server/io.c
@@ -59,7 +59,9 @@ static void write_value_as_line(uint32_t value, FILE * file);
 static void write_inventory(struct Thing * player, FILE * file);
 
 /* Return map cells sequence as visible to the "player", with invisible cells as
  * whitespace. Super-impose over visible map cells things positioned there.
  */
 static char * build_visible_map(struct Thing * player);
 
@@ -314,20 +316,22 @@ static char * build_visible_map(struct Thing * player)
             }
         }
         struct Thing * t;
-        struct ThingType * tt;
         char c;
         uint8_t i;
-        for (i = 0; i < 2; i++)
+        for (i = 0; i < 3; i++)
         {
             for (t = world.things; t != 0; t = t->next)
             {
-                if (   'v'==player->fov_map[t->pos.y*world.map.length+t->pos.x]
-                    && (   (0 == i && 0 == t->lifepoints)
-                        || (1 == i && 0 < t->lifepoints)))
+                if ('v'==player->fov_map[t->pos.y*world.map.length+t->pos.x])
                 {
-                    tt = get_thing_type(t->type);
-                    c = tt->char_on_map;
-                    visible_map[t->pos.y * world.map.length + t->pos.x] = c;
+                    struct ThingType * tt = get_thing_type(t->type);
+                    if (   (0 == i && !t->lifepoints && !tt->consumable)
+                        || (1 == i && !t->lifepoints &&  tt->consumable)
+                        || (2 == i &&  t->lifepoints))
+                    {
+                        c = tt->char_on_map;
+                        visible_map[t->pos.y * world.map.length + t->pos.x] = c;
+                    }
                 }
             }
         }
@@ -59,7 +59,9 @@ static void write_value_as_line(uint32_t value, FILE * file);
 
 /* Return map cells sequence as visible to the "player", with invisible cells as
- * whitespace. Super-impose over visible map cells things positioned there.
+ * whitespace. Super-impose over visible map cells things positioned there,
+ * with animate things overwriting inanimate things, and inanimate consumable
+ * things overwriting inanimate non-consumable things.
  */
 static char * build_visible_map(struct Thing * player);